import _extends from "@babel/runtime/helpers/esm/extends";

// TYPES
// FUNCTIONS
function serializePositiveNumber(value) {
  return value === Infinity ? -1 : value;
}

function deserializePositiveNumber(value) {
  return value === -1 ? Infinity : value;
} // Most config is not dehydrated but instead meant to configure again when
// consuming the de/rehydrated data, typically with useQuery on the client.
// Sometimes it might make sense to prefetch data on the server and include
// in the html-payload, but not consume it on the initial render.


function dehydrateQuery(query) {
  return {
    cacheTime: serializePositiveNumber(query.cacheTime),
    state: query.state,
    queryKey: query.queryKey,
    queryHash: query.queryHash
  };
}

function defaultShouldDehydrate(query) {
  return query.state.status === 'success';
}

export function dehydrate(client, options) {
  options = options || {};
  var shouldDehydrateQuery = options.shouldDehydrateQuery || defaultShouldDehydrate;
  var queries = [];
  client.getQueryCache().getAll().forEach(function (query) {
    if (shouldDehydrateQuery(query)) {
      queries.push(dehydrateQuery(query));
    }
  });
  return {
    queries: queries
  };
}
export function hydrate(client, dehydratedState, options) {
  if (typeof dehydratedState !== 'object' || dehydratedState === null) {
    return;
  }

  var queryCache = client.getQueryCache();
  var queries = dehydratedState.queries || [];
  queries.forEach(function (dehydratedQuery) {
    var query = queryCache.get(dehydratedQuery.queryHash); // Do not hydrate if an existing query exists with newer data

    if (query) {
      if (query.state.updatedAt < dehydratedQuery.state.updatedAt) {
        query.setState(dehydratedQuery.state);
      }

      return;
    } // Restore query


    queryCache.build(client, _extends({}, options == null ? void 0 : options.defaultOptions, {
      queryKey: dehydratedQuery.queryKey,
      queryHash: dehydratedQuery.queryHash,
      cacheTime: deserializePositiveNumber(dehydratedQuery.cacheTime)
    }), dehydratedQuery.state);
  });
}