import React from 'react';
import { useIsMounted } from './utils';
import { useQueryErrorResetBoundary } from './QueryErrorResetBoundary';
import { useQueryClient } from './QueryClientProvider';
export function useBaseQuery(options, Observer) {
  var queryClient = useQueryClient();
  var isMounted = useIsMounted();
  var errorResetBoundary = useQueryErrorResetBoundary();
  var defaultedOptions = queryClient.defaultQueryObserverOptions(options); // Always set stale time when using suspense

  if (defaultedOptions.suspense && !defaultedOptions.staleTime) {
    defaultedOptions.staleTime = 2000;
  } // Create query observer


  var observerRef = React.useRef();
  var observer = observerRef.current || new Observer(queryClient, defaultedOptions);
  observerRef.current = observer; // Update options

  if (observer.hasListeners()) {
    observer.setOptions(defaultedOptions);
  }

  var _React$useState = React.useState(function () {
    return observer.getCurrentResult();
  }),
      currentResult = _React$useState[0],
      setCurrentResult = _React$useState[1]; // Subscribe to the observer


  React.useEffect(function () {
    errorResetBoundary.clearReset();
    return observer.subscribe(function (result) {
      if (isMounted()) {
        setCurrentResult(result);
      }
    });
  }, [isMounted, observer, errorResetBoundary]); // Handle suspense

  if (observer.options.suspense || observer.options.useErrorBoundary) {
    if (currentResult.isError && !errorResetBoundary.isReset() && !observer.getCurrentQuery().isFetching()) {
      throw currentResult.error;
    }

    if (observer.options.suspense && !observer.hasListeners() && observer.willFetchOnMount()) {
      errorResetBoundary.clearReset();
      throw observer.getNextResult({
        throwOnError: true
      });
    }
  }

  return currentResult;
}