"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.useMutation = useMutation;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _utils = require("./utils");

var _utils2 = require("../core/utils");

var _mutationObserver = require("../core/mutationObserver");

var _QueryClientProvider = require("./QueryClientProvider");

function useMutation(arg1, arg2, arg3) {
  var options = (0, _utils2.parseMutationArgs)(arg1, arg2, arg3);
  var isMounted = (0, _utils.useIsMounted)();
  var queryClient = (0, _QueryClientProvider.useQueryClient)(); // Create mutation observer

  var observerRef = _react.default.useRef();

  var observer = observerRef.current || new _mutationObserver.MutationObserver(queryClient, options);
  observerRef.current = observer; // Update options

  if (observer.hasListeners()) {
    observer.setOptions(options);
  }

  var _React$useState = _react.default.useState(function () {
    return observer.getCurrentResult();
  }),
      currentResult = _React$useState[0],
      setCurrentResult = _React$useState[1]; // Subscribe to the observer


  _react.default.useEffect(function () {
    return observer.subscribe(function (result) {
      if (isMounted()) {
        setCurrentResult(result);
      }
    });
  }, [observer, isMounted]);

  var mutate = _react.default.useCallback(function (variables, mutateOptions) {
    observer.mutate(variables, mutateOptions).catch(_utils2.noop);
  }, [observer]);

  if (currentResult.error && observer.options.useErrorBoundary) {
    throw currentResult.error;
  }

  return (0, _extends2.default)({}, currentResult, {
    mutate: mutate,
    mutateAsync: currentResult.mutate
  });
}